/*
 *  2004 Michal 'GiM' Spadlinski <gim at skrzynka dot pl>
 *
 *  This is simple stat() command presentation
 *  + does lstat() on file
 *  + prints some information about file
 * 
 */
/* this should compile on Solaris, Linux, FBSD */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <fcntl.h>
#include <time.h>

int main(int argc, char **argv)
{
	struct stat st;
	struct passwd *p;
	struct group *g;
	mode_t m, fm;
	int fd, ds;
	time_t at, mt, ct;
	char *t[3], form[30]; 

	if (argc != 2)
	{
		fprintf(stderr, "Syntax) ./name filename\n");
		return -1;
	}
	
	if (lstat(*++argv, &st) == -1)
	{
		perror("stat()");
		return -1;
	}

	m = st.st_mode;
	switch(fm=m&S_IFMT)
	{
		case S_IFSOCK:
			printf("   socket"); break;
		case S_IFLNK:
			printf("  symlink"); break;
		case S_IFREG:
			printf("  regular"); break;
		case S_IFBLK:
			printf(    "block"); break;
		case S_IFDIR:
			printf("directory"); break;
		case S_IFCHR:
			printf("  chardev"); break;
		case S_IFIFO:
			printf("     fifo"); break;
		default:
			printf("  unknown"); 
	}
	printf ("%o\t",st.st_mode);

	fputc(m&S_IRUSR?'r':'-', stdout);
	fputc(m&S_IWUSR?'w':'-', stdout);
	if (m&S_IXUSR)	fputc(m&S_ISUID?'s':'x', stdout);
	else		fputc(m&S_ISUID?'S':'-', stdout);

	fputc(m&S_IRGRP?'r':'-', stdout);
	fputc(m&S_IWGRP?'w':'-', stdout);
	if (m&S_IXGRP)	fputc(m&S_ISGID?'s':'x', stdout);
	else		fputc(m&S_ISGID?'S':'-', stdout);
	
	fputc(m&S_IROTH?'r':'-', stdout);
	fputc(m&S_IWOTH?'w':'-', stdout);
	if (m&S_IXOTH)	fputc(m&S_ISVTX?'t':'x', stdout);
	else		fputc(m&S_ISVTX?'T':'-', stdout);

	printf("\tinode: %lu\n", st.st_ino); 
	ds = sizeof(dev_t);
	sprintf(form, "dev: %%0%d%sX %%%su", ds<<1, ds>4?"ll":"l", ds>4?"ll":"l");
	printf(form, st.st_dev, st.st_dev);
	if (fm == S_IFBLK || fm == S_IFCHR)
		printf("\trdev: %lu %08X", st.st_rdev, st.st_rdev);
	puts("");

	/* uid: username (uid) gid: grpname (gid) */
	printf("uid: ");
	if (p = getpwuid(st.st_uid))
		printf(p->pw_name);
	else
		printf("cannot_retrive");
	printf(" (%ld) gid: ", st.st_uid);
	if (g = getgrgid(st.st_gid))
		printf(g->gr_name);
	else
		printf("cannot_retrive");
	printf(" (%ld)", st.st_gid);
	/*if (st.st_nlink>1) */
		printf(" hardlinks#: %ld", st.st_nlink);

	printf("\nsize: %ld\tblocks: %ld\toptimalblock: %ld\n",
			st.st_size, st.st_blocks, st.st_blksize);
	ct = st.st_ctime;
	mt = st.st_mtime;
	at = st.st_atime;
	t[0] = strdup((const char *)ctime(&ct));
	t[1] = strdup((const char *)ctime(&mt));
	t[2] = strdup((const char *)ctime(&at));
	printf("      Change: %sModification: %s      Access: %s\n",
			t[0], t[1], t[2]);
	free(t[0]);
	free(t[1]);
	free(t[2]);
	return -1;
}
